#ifndef ACQUIREFILE_READ_H
#define ACQUIREFILE_READ_H

#include "acquirefile.h"
#include "AcquireFormat.h"

/* AcquireFormat.h defines the fixed-size structures;
   here we define variable-length aggregates corresponding to one or more blocks.
   Also, functions to open, close, and read sampled data.  The struct acquirefile_read
   represents an open .acquire file.
*/

struct acquirefile_read; /* top-level, below */

struct acquirefile_read_channel
{
  AcquireChannelDescriptor descriptor;
  AcquireBlockDescriptor *blocks;
};
#ifndef __cplusplus
typedef struct acquirefile_read_channel acquirefile_read_channel;
#endif

struct acquirefile_read_segment
{
  AcquireSegmentDescriptor descriptor;
  acquirefile_read_channel *channels;
  int64_t frames; /* total samples (frames) this segment; calculated as sum(block.descriptor.data_count) for all blocks in channel 0.
                                                          [TODO: is this reliably correct? can it vary among channels?] */
};
#ifndef __cplusplus
typedef struct acquirefile_read_segment acquirefile_read_segment;
#endif

struct acquirefile_read_sweep
{
  AcquireSweepDescriptor descriptor;
  AcquireSweepPrefixAnalogChannel *analog;
  AcquireSweepPrefixAperiodicChannel *aperiodic;
  int64_t control_channel_count;
  AcquireBlockDescriptor *control_channels;
  int64_t control_channel_ext_count;
  AcquireBlockDescriptor *control_channels_ext;
  int64_t *segment_directory;
  int64_t *channel_directory;
  acquirefile_read_segment *segments;
};
#ifndef __cplusplus
typedef struct acquirefile_read_sweep acquirefile_read_sweep;
#endif

struct acquirefile_read_series
{
  AcquireSeriesDescriptor descriptor;
  int64_t *sweep_directory;
  acquirefile_read_sweep **sweeps;
};
#ifndef __cplusplus
typedef struct acquirefile_read_series acquirefile_read_series;
#endif

struct acquirefile_read
{
  AcquirePrefix prefix;
  AcquireHeader header;
  AcquireAnalogInChannelRecord *analog_in;
  AcquireAperiodicChannelRecord *aperiodic;
  AcquireBlockTag control_tag;
  void *control_block;
  acquirefile_read_series *series;
  void *storage; /* opaque handle to open file */
  int8_t *data_block_buf;
  int16_t *data_buf;
};
#ifndef __cplusplus
typedef struct acquirefile_read acquirefile_read;
#endif

#ifdef __cplusplus
extern "C" {
#endif

ACQUIREFILE_API acquirefile_read *acquirefile_read_open(const char *path);
#if defined(_WIN32) && defined(UNICODE)
ACQUIREFILE_API acquirefile_read *acquirefile_read_open_w(const _TCHAR *path);
#endif
ACQUIREFILE_API void acquirefile_read_close(acquirefile_read *file);

ACQUIREFILE_API int64_t acquirefile_read_data_int16(acquirefile_read *file,
						    int64_t series, int64_t sweep, int64_t segment, int64_t channel,
						    int64_t offset, int64_t count, int16_t *buf);

ACQUIREFILE_API int64_t acquirefile_read_data_float(acquirefile_read *file,
						    int64_t series, int64_t sweep, int64_t segment, int64_t channel,
						    int64_t offset, int64_t count, float *buf);

#ifdef __cplusplus
} // extern "C"
#endif

#endif
